package de.halbekunst.juplo.cachecontrol.annotations;


import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;


/**
 * Marker-Annotation für Handler (i.A. eine Impelementierung von
 * {@link Controller}), deren Antworten vom {@link CachingInterceptor} mit
 *  HTTP/1.1-Caching-Header nach
 * {@linkplain http://www.w3.org/Protocols/rfc2616/rfc2616-sec13.html RFC 2616}
 * dekoriert werden sollen.
 * <p>
 * Wenn der Parameter <code>eager</code> auf <code>true</code> gesetzt wird,
 * ermittelt der {@link CachingInterceptor} die Cache-Parameter über die
 * annotierten Methoden vorab.
 * <strong>Achtung:</strong>
 * Dies bedeutet, dass die annotierten Methoden aufgerufen werden <em>bevor</em>
 * die eigentliche Verarbeitungs-Routine der markierten Klasse aufgerufen wird!
 * Wenn sich dabei ergiebt, dass die Antwort nicht erneut ausgeliefert werden
 * muss, wird die eigentliche Verarbeitungs-Routine <em>gar nicht aufgerufen</em>.
 * <p>
 * Wenn der Parameter <code>eager</code> nicht gesetzt ist (oder explizit auf
 * <code>false</code> gesetzt wurde), kapselt der {@link CachingInterceptor}
 * den Request und den Ausgabestrom für den Response-Body und trifft die
 * Entscheidung über die zu ergänzenden Header, wenn der Status des
 * {@link HttpServletResponse} gesetzt oder mit dem Schreiben des Response-Body
 * begonnen wird.
 *
 * @see CacheControl
 * @see Accepts
 * @see CacheSeconds
 * @see LastModified
 * @see ETag
 * @see CacheControl
 * @author kai
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface Cacheable {
  boolean eager() default false;
}
